#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    Shimehari.configuration
    ~~~~~~~~~~~~~~~~~~~~~~~

    アプリケーション内の様々な設定や環境情報を保持する
    Config クラスを提供します。
    
===============================
"""

ENVIROMENTS = ['development', 'production']



u"""
===============================
    ::pkg:: Shimehari.coniguration

    ConfigManager
    ~~~~~~~~~~~~~
    Config インスタンスを管理します。
    
===============================
"""
class ConfigManager(object):
    configrations = {}



    u"""-----------------------------
        [[classmethod]]
        ::pkg:: Shimehari.configrations.ConfigManager
        hasConfig
        ~~~~~~~~~

        Config インスタンスが登録されているかどうか
        [return]
            :bool 登録されているかどうか
    ------------------------------"""
    @classmethod
    def hasConfig(cls):
        if not cls.configrations or len(cls.configrations) < 1:
            return False
        else:
            return True


    u"""-----------------------------
        [[classmethod]]
        ::pkg:: Shimehari.configrations.ConfigManager
        hasConfig
        ~~~~~~~~~

        Config インスタンスが登録されているかどうか
        [return]
            :bool 登録されているかどうか
    ------------------------------"""
    @classmethod
    def getConfigs(cls):
        return cls.configrations



    u"""-----------------------------
        [[classmethod]]
        ::pkg:: Shimehari.configrations.ConfigManager
        getConfig
        ~~~~~~~~~

        Config インスタンスを取得します
        [args]
            :environ キーとなる環境
        [return]
            :Config
    ------------------------------"""
    @classmethod
    def getConfig(cls, environ):
        if environ in cls.configrations:            
            return cls.configrations[environ]
        return None



    u"""-----------------------------
        [[classmethod]]
        ::pkg:: Shimehari.configrations.ConfigManager
        addConfigs
        ~~~~~~~~~~

        Config インスタンスをまとめて登録します
        [args]
            :configs Config インスタンスを格納したディクショナリ

    ------------------------------"""
    @classmethod
    def addConfigs(cls,configs=[]):
        [cls.addConfig(c) for c in configs]



    u"""-----------------------------
        [[classmethod]]
        ::pkg:: Shimehari.configrations.ConfigManager
        addConfig
        ~~~~~~~~~

        Config インスタンスを登録します。
        [args]
            :config 登録したいコンフィグインスタンス
            
    ------------------------------"""
    @classmethod
    def addConfig(cls, config):
        if isinstance(config, Config):
            cls.configrations.setdefault(config.environment,config)
        else:
            raise TypeError('config じゃない')


    @classmethod
    def removeConfig(cls,environment):
        if cls.configrations[environment]:
            cls.configrations.pop(environment)


    """0.2
    @classmethod
    def current(cls):
        return config
    """


u"""
===============================
    ::pkg:: Shimehari.coniguration
    
    Config
    ~~~~~~
    アプリケーションな設定や環境情報を保持する
    コンフィグクラス
    
===============================
"""
from datetime import timedelta
class Config(dict):
    defaults ={
        'DEBUG':False,
        'TEST':False,
        'APP_ROOT':None,
        'APP_DIRECTORY':'app',
        'MAIN_SCRIPT':'main',
        'APP_INSTANCE_NAME':'app',
        'CONTROLLER_DIRECTORY':'controllers',
        'VIEW_DIRECTORY':'views',
        'MODEL_DIRECTORY':'models',
        'ASSETS_DIRECTORY':'assets',
        'PREFERRED_URL_SCHEME':'http',
        'AUTO_SETUP':True,
        'TEMPLATE_ENGINE':'jinja2',
        'USE_X_SENDFILE':False,
        'SECRET_KEY':'_secret_shimehari',
        'SERVER_NAME':None,
        'TRAP_HTTP_EXCEPTIONS':False,
        'TRAP_BAD_REQUEST_ERRORS':False,
        'PRESERVE_CONTEXT_ON_EXCEPTION':None,
        'SEND_FILE_MAX_AGE_DEFAULT': 12 * 60 * 60,
        'PERMANENT_SESSION_LIFETIME':timedelta(days=31)
    }
    def __init__(self, environment='development', defaults={}):
        d = self.defaults.copy()
        d.update(defaults)
        dict.__init__(self, d)
        self.environment = environment





#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    Shimehari.core.manage.commands.generate
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    アプリケーションモジュールを新たに作成する generate コマンド

    各コマンドモジュールは共通インターフェースとして
    Command クラスを持ちます。
    
===============================
"""
import os
import sys
import shutil

from optparse import make_option

import shimehari
from shimehari.core.exceptions import CommandError
from shimehari.core.helpers import importFromString
from shimehari.core.manage import CreatableCommand
from shimehari.helpers import getEnviron
from shimehari.configuration import ConfigManager, Config

u"""
===============================
    ::pkg:: Shimehari.core.manage.commands.generate
    Command
    ~~~~~~~
    
    コマンドの実装
    
===============================
"""
class Command(CreatableCommand):
    help = ("Generate Shimehari Modules")
    option_list = CreatableCommand.option_list + (
            make_option('--path', '-p', action='store', type='string', dest='path', help='generating target path'),
        )

    def handle(self, moduleType, name, *args, **options):
        if not moduleType == 'controller':
            raise CommandError('ない')
        
        path = options.get('path')
        if path is None:
            currentPath = os.getcwd()
            try:
                importFromString('config')
            except:
                raise CommandError('config file is not found...')
            config = ConfigManager.getConfig(getEnviron())
            path = os.path.join(currentPath, config['APP_DIRECTORY'], config['CONTROLLER_DIRECTORY'])

        if not os.path.isdir(path):
           raise CommandError('Given path is invalid')

        ctrlTemplate = os.path.join(shimehari.__path__[0], 'core','conf', 'controller_template.py')
        
        name, filename = self.filenameValidation(name)
        newPath = os.path.join(path,filename)

        self.readAndCreateFileWithRename(ctrlTemplate, newPath, name)



    def filenameValidation(self, name):
        if name.endswith(('.pyc','.pyo', '.py.class')):
            raise CommandError('invalid name....')
        if not name.endswith('.py'):
            filename = name + '.py'
        else:
            filename = name
            name = name.replace('.py', '')

        import re
        if re.search(r"\W", name) or re.match(r"\d", name[0]):
            raise CommandError('file name is invalid')
        name = name[0].upper() + name[1:]
        return name, filename



    u"""-----------------------------
        ::pkg:: Shimehari.core.manage.commands.generate.Command
        readAndCreateFileWithRename
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~

        指定されたディレクトリからテンプレートファイルを読み込み
        新たに生成したい指定ディレクトリへファイルを生成します。
        [args]
            :old テンプレートファイルのパス
            :new 生成したいディレクトリへのパスとファイル名
            :name 変更したいクラス名
    ------------------------------"""
    def readAndCreateFileWithRename(self, old, new, name):
        if os.path.exists(new):
            raise CommandError('Controller already exists.')

        with open(old, 'r') as template:
            content = template.read()
            if '%s' in content:
                content = content % name
        with open(new, 'w') as newFile:
            newFile.write(content)
        sys.stdout.write("Genarating New Controller: %s\n" % new)

        try:
            shutil.copymode(old,new)
            self.toWritable(new)
        except OSError:
            sys.stderr.write('can not setting permission')



# -*- coding: utf-8 -*-
#!/usr/bin/env python


u"""
======================================
    Shimehari Command Line Manager
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    ochoko is Shimehari Command Line Manager.

    commands example....

    :new <app_name> [options]...
        create new Shimehari App.

    :generate <controller_name> [options]....
        generate new controller

        ::Options

        -p, [--path]:
            path shitei


    and see more --help

======================================
"""

import os,sys

if __name__ == '__main__':
    from shimehari.core.manage import executeFromCommandLine
    executeFromCommandLine(sys.argv)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    [[Shimehari Config]]
    config
    ~~~~~~
    
===============================
"""

from datetime import timedelta
from shimehari.configuration import Config, ConfigManager

ConfigManager.addConfigs([
    Config('development', {
        'DEBUG':False,

        'TEST':False,

        #アプリのディレクトリ名
        'APP_DIRECTORY':'%s',

        'MAIN_SCRIPT':'main',

        #アプリケーションinstanceの名前
        'APP_INSTANCE_NAME':'app',

        'CONTROLLER_DIRECTORY':'controllers',

        'VIEW_DIRECTORY':'views',

        #for daiginjou
        'MODEL_DIRECTORY':'models',

        'PREFERRED_URL_SCHEME':'http',

        'AUTO_SETUP':True,

        'TEPMLATE_ENGINE':'jinja2',

        'SECRET_KEY':'_secret_shimehari',

        'SERVER_NAME':None,

        'PRESERVE_CONTEXT_ON_EXCEPTION':None,

        'PERMANENT_SESSION_LIFETIME':timedelta(days=31)
    })
])


#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    ::pkg:: Shimehari.core.command
    AbstractCommand
    ~~~~~~~~~~~~~~~
    コマンドラインツール

    抽象クラス
    Command の元となるクラス
===============================
"""

import sys
from optparse import OptionParser, make_option
import traceback
import shimehari

from shimehari.core.exceptions import CommandError

class AbstractCommand(object):
    option_list = (
            make_option('--traceback', action='store_true', help='Print traceback on exception'),
        )
    help = ''
    args = ''

    def getVersion(self):
        return shimehari.getVersion()

    def usage(self, subcommand):
        usage = '%%prog %s [options] %s' % (subcommand, self.args)
        if self.help:
            return '%s\n\n%s' %(usage, self.help)
        else:
            return usage

    def createParser(self, progName, subcommand):
        return OptionParser(prog=progName, 
                            usage=self.usage(subcommand),
                            version=self.getVersion(),
                            option_list=self.option_list)

    def runFromArgv(self, argv):
        parser = self.createParser(argv[0], argv[1])
        options, args = parser.parse_args(argv[2:])
        self.execute(*args, **options.__dict__)

    def execute(self, *args, **options):
        
        showTraceback = options.get('traceback', False)

        try:
            self.stdout = options.get('stdout', sys.stdout)
            self.stderror = options.get('stderr', sys.stderr)

            output = self.handle(*args,**options)
            if output:
                self.stdout.write(output)
        except (TypeError,CommandError), e:
            if showTraceback:
                traceback.print_exc()
            else:
                self.stderror.write('%s\n' % unicode(e).encode('utf-8'))
            sys.exit(1)

    def validate(self):
        raise NotImplementedError()

    def handle(self):
        raise NotImplementedError()

    #humu...
    def getAppConfig(self):
        pass

import os

class CreatableCommand(AbstractCommand):

    u"""-----------------------------
        ::pkg:: Shimehari.core.manage.AbstractCommand.CreatableCommand
        toWritable
        ~~~~~~~~~~

        ファイルを書き込み可能状態にします。
        [args]
            :filename 対象ファイル名
    ------------------------------"""
    def toWritable(self,filename):
        if sys.platform.startswith('java'):
            return

        if not os.access(filename, os.W_OK):
            st = os.stat(filename)
            newPermission = stat.S_IMODE(st.st_mode)
            os.chmod(filename, newPermission)




#!/usr/bin/env python
# -*- cofing: utf-8 -*-

from shimehari import Router, Resource
from controllers import IndexController

#
# You can routing like Rails style.
#
#app.router = RESTfulRouter(
#        Resource(IndexController,root=True),
#        Resource(SomeOtherController),
#        Resource(SomethingElseController, expect=['edit', 'destroy'])
#    )


appRoutes = Router([
		Resource(IndexController, root=True)
	])

#
# You can routing like werkzeug style.
# 
# from shimehari import BasicRouter
# app.router = BasicRouter(
#        {'/':IndexController.list},
#        {'/hoge':SomeOtherController.list},
#        {'/hoge/<int:id>':SomeOtherController.show}
#    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    Shimehari.core.manage.commands.routes
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    現在のルーティング状況をコマンドラインへダンプします。
    
===============================
"""

import os
import sys

from shimehari.core.exceptions import CommandError
from shimehari.core.manage import AbstractCommand
from shimehari.core.helpers import importFromString
from shimehari.configuration import ConfigManager, Config
from shimehari.helpers import getEnviron

class Command(AbstractCommand):
	def handle(self, *args, **options):
		try:
			importFromString('config')
		except ImportError:
			raise CommandError('コンフィグファイルが見当たりません')

		config = ConfigManager.getConfig(getEnviron())
		appPath = os.path.join(os.getcwd(), config['APP_DIRECTORY'])
		if not os.path.isdir(appPath):
			raise CommandError('アプリケーションが見当たりません')

		try:
			router = importFromString(config['APP_DIRECTORY'] + '.' + 'router')
		except Exception, e:
			raise CommandError(u'ルーターがみつかりません。\n %s' % e)
		
		sys.stdout.write('\nYour Shimehari App Current Routing.\n\n')
		sys.stdout.write('Methods       |URL                          |Action\n')
		sys.stdout.write('----------------------------------------------------------------------\n')
		sys.stdout.write(router.appRoutes.dump())
		sys.stdout.write('\n')





#!/usr/bin/env python
# -*- coding: utf-8 -*-

class ShimehariException(Exception):
    def __init__(self, description):
        self.description = description

    def __str__(self):
        return 'ShimehariError: %s' % self.description



class ShimehariHTTPError(ShimehariException):
    def __init__(self,description, host='127.0.0.1', port=5959):
        ShimehariException.__init__(description)



class ShimehariSetupError(ShimehariException):
    def __str__(self):
        return 'ShimehariSetupError: %s' % self.description


class CommandError(Exception):pass
    

class DrinkError(Exception):
    def __init__(self, description, host=None, port=None):
        self.description = description
        self.host = host
        self.port = port

    def __str__(self):
        return repr(self)

    def __repr__(self):
        return 'Shimehari Drink Command is Error: %s\n host=%s\n port=%d' % (self.description, self.host, self.port)

from werkzeug.exceptions import HTTPException, BadRequest
from shimehari.helpers import json
class JSONHTTPExceptions(HTTPException):
    def get_body(self,environ):
        return json.dumps(dict(description=self.get_description(environ)))

    def get_headers(self, environ):
        return [('Content-Type','application/json')]

class JSONBadRequest(JSONHTTPExceptions, BadRequest):
    description = ('ブラウザ、もしくはプロクシが送ったリクエストを、このアプリケーションでは処理できません。')


#!/usr/bin/env python
# -*- coding: utf-8 -*-
u"""
===============================
    Shimehari.app
    ~~~~~~~~~~~~~
    Flask などなど参考に。
    ルーティングが多少複雑になっても
    対応できるような作りにしたいなぁ
===============================
"""

import os
import sys
from  threading import Lock, RLock
from functools import update_wrapper
from datetime import timedelta

from werkzeug.exceptions import NotFound, HTTPException, InternalServerError, MethodNotAllowed, BadRequest
from werkzeug.datastructures import ImmutableDict
from werkzeug.routing import RequestRedirect, Rule

from .helpers import _Kouzi, findPackage, getHandlerAction, getModulesFromPyFile, getEnviron, \
                     lockedCachedProperty, getTemplater
from .contexts import RequestContext, AppContext
from .routing import Router
from core.config import RESTFUL_ACTIONS
from .wrappers import Request, Response
from shimehari.configuration import Config, ConfigManager
from shimehari.session import SessionStore
from shimehari.shared import _requestContextStack, _appContextStack, _SharedRequestClass, request
from shimehari.template import _defaultTemplateCtxProcessor
from shimehari.core.exceptions import ShimehariException, ShimehariSetupError
from shimehari.core.signals import appContextTearingDown, requestContextTearingDown


_loggerLock = Lock()

def setupMethod(f):
    def wrapperFunc(self, *args,**kwargs):
        if self.debug or self._gotFirstRequest:
            raise AssertionError('Setup seems to have already completed ...')
        return f(self, *args, **kwargs)
    return update_wrapper(wrapperFunc, f)


class Shimehari(_Kouzi):

    currentEnv = getEnviron()
    debug = None
    testing = None

    debugLogFormat = ('-' * 80 + '\n' +
        '%(levelname)s in %(module)s [%(pathname)s:%(lineno)d]:\n' +
        '%(message)s\n' +
        '-' * 80)

    requestClass = Request
    responseClass = Response

    testClientCls = None

    teardownAppContextFuncs = []

    defaultConfig = {
        'DEBUG':False,
        'TEST':False,
        'APP_DIRECTORY':'app',
        'CONTROLLER_DIRECTORY':'controllers',
        'VIEW_DIRECTORY':'views',
        #for daiginjou
        'MODEL_DIRECTORY':'models',
        'PREFERRED_URL_SCHEME':'http',
        'AUTO_SETUP':True,
        'TEMPLATE_ENGINE':'jinja2',
        'TRAP_HTTP_EXCEPTIONS':False,
        'SERVER_NAME':None,
        'PERMANENT_SESSION_LIFETIME':timedelta(days=31),
        'SECRET_KEY':'_secret_shimehari'
    }

    #tempalte options

    sessionStore = SessionStore()

    sharedRequestClass = _SharedRequestClass

    def __init__(self, importName, 
                 staticURL=None, staticFolder='static',
                 appFolder='app', controllerFolder='controllers', 
                 viewFolder='views', assetsFolder='assets',
                 instancePath=None, isRelativeConfig=False):

        _Kouzi.__init__(self, importName, appFolder=appFolder, 
                        controllerFolder=controllerFolder, viewFolder=viewFolder)
        
        if instancePath is None:
            self._instancePath = self.getInstancePath()

        self._logger = None
        self.loggerName = self.importName

        self.config = self.getConfig()


        self.controllers = {}
        self.urlValuePreprocesors = {}
        self.beforeRequestFuncs = {}
        self.beforeFirstRequestFuncs = []
        self.urlDefaultFuncs = {}
        self.afterRequestFuncs = {}
        self._errorHandlers = {}
        self.errorHandlerSpec = {None:self._errorHandlers}
        self.buildErrorHandlers = None
        self.teardownRequestContextFuncs = {}

        self.templateContextProcessors = {
            None: [_defaultTemplateCtxProcessor]
        }

        #CSRF
        from shimehari.crypt import CSRF
        self.csrf = CSRF(self)

        self._router = Router()

        self._gotFirstRequest = False

        self._beforeRequestLock = Lock()

        self.debug = self.config['DEBUG']
        self.test = self.config['TEST']
        self.sessionKey = self.config['SECRET_KEY']
        self.useXSendFile = self.config['USE_X_SENDFILE']

        #ふむ
        self.templater = getTemplater(self, self.config['TEMPLATE_ENGINE'])

        if self.config['AUTO_SETUP']:
            self.setup()


    @property
    def gotFirstRequest(self):        
        return self._gotFirstRequest



    @property
    def propagateExceptions(self):
        return self.testing or self.debug



    @property
    def preserveContextOnException(self):
        rv = self.config['PRESERVE_CONTEXT_ON_EXCEPTION']
        if rv is not None:
            return rv
        return self.debug



    #@property
    #def useXSendFile(self):
    #    return self.config['USE_X_SENDFILE']



    @property
    def logger(self):
        if self._logger and self._logger.name == self.loggerName:
            return self._logger
        with _loggerLock:
            if self._logger and self._logger.name == self.loggerName:
                return self._logger
            from shimehari.logging import createLogger
            self._logger = rv = createLogger(self)
            return rv


    u"""--------------------------------------
        ルーティング
    --------------------------------------"""
    def router():
        doc = "The router property."
        def fget(self):
            return self._router
        def fset(self, value):
            self.setControllerFromRouter(value)
            self._router = value
        def fdel(self):
            self.controllers = {}
            del self._router
        return locals()
    router = property(**router())



    """
    ===============================
        methods
    ===============================
    """

    u"""-----------------------------
        ::pkg:: Shimehari.app
        getInstancePath
        ~~~~~~~~~~~~~~~

        インスタンスパスを返します。
        [return]
            :str インスタンスパス
    ------------------------------"""
    def getInstancePath(self):
        prefix, pkgPath = findPackage(self.importName)
        if prefix is None:
            return os.path.join(pkgPath, 'instance')
        return os.path.join(prefix, 'var', self.name + '-instance')



    u"""-----------------------------
        ::pkg:: Shimehari.app
        getConfig
        ~~~~~~~~~

        現在の コンフィグ を返します。
        [return]
            :Config 現在のコンフィグ
    ------------------------------"""
    def getConfig(self):
        configs = ConfigManager.getConfigs()
        try:
            from .config import config
            configs = ConfigManager.getConfigs()
        except ImportError:
            pass
        if not configs:
            cfg = Config(self.currentEnv, self.defaultConfig)
            ConfigManager.addConfig(cfg)
            return cfg
        else:
            return configs[self.currentEnv]



    def saveSession(self, session, response):
        if session.should_save:
            self.sessionStore.save(session, response)
            response.set_cookie(self.sessionKey, session.sid)
        return response



    def openSession(self, request):
        sid = request.cookies.get(self.sessionKey, None)
        if sid is None:
            return self.sessionStore.new()
        else:
            return self.sessionStore.get(sid)


    u"""-----------------------------
        ::pkg:: Shimehari.app
        setControllerFromRouter
        ~~~~~~~~~~~~~~~~~~~~~~~

        ルーターからコントローラーを設定します。
        [args]
        :router
            るーたー
    ------------------------------"""
    def setControllerFromRouter(self, router):
        if not self.controllers:
            self.controllers = {}
        for rule in router._rules:
            self.controllers[rule.endpoint] = rule.endpoint



    u"""-----------------------------
        ::pkg:: Shimehari.app
        addController
        ~~~~~~~~~~~~~

        [args]
        :controller
            追加したいコントローラー
            追加されたコントローラーは
            アプリケーションの管理下に置かれ、
            ルーティングが自動生成されます。
    ------------------------------"""
    def addController(self, controller):
        for action in RESTFUL_ACTIONS:
            handler = getHandlerAction(controller, action)
            if handler is not None:
                self.controllers[handler] = handler



    def addRoute(self, url, func, methods=None, **options):
        rule = Rule(url, endpoint=func.__name__, methods=methods)
        self.controllers[func.__name__] = func
        self.router.add(rule)



    def logException(self, excInfo):
        self.logger.error('excepts on %s [%s]' % (request.path, request.method), exc_info=excInfo)



    def injectURLDefaults(self, endpoint, values):
        funcs = self.urlDefaultFuncs.get(None,())
        for func in funcs:
            func(endpoint, vlaues)



    @lockedCachedProperty
    def templateLoader(self):
        return self.templater.templateLoader



    @lockedCachedProperty
    def templateEnv(self):
        rv = self.templater.templateEnv()
        return rv



    def createTemplateEnvironment(self):
        rv = self.templater.createTemplateEnvironment()
        return rv



    def createGlobalTemplateLoader(self):
        return self.templater.dispatchLoader(self)



    def updateTemplateContext(self,context):
        self.templater.updateTemplateContext(context)



    u"""-----------------------------
        ::pkg:: Shimehari.app
        setup
        ~~~~~

        アプリケーションをセットアップします。
        指定された app ディレクトリ配下にある
        コントローラー、ルーターを探し出しバインドします。

    ------------------------------"""
    def setup(self):
        self.appPath = os.path.join(self.rootPath, self.appFolder)
        if not os.path.isdir(self.appPath):
            raise ShimehariSetupError('Application directory is not found\n%s' % self.rootPath)
            sys.exit(0)
        try:
            apps = __import__(self.appFolder)
            self.setupBindController()
            self.setupBindRouter()
        except (ImportError, AttributeError):
            raise ShimehariSetupError('Application directory is invalid')



    u"""-----------------------------
        ::pkg:: Shimehari.app
        setupBindController
        ~~~~~~~~~~~~~~~~~~~

        コントローラーをバインドします
        
    ------------------------------"""
    def setupBindController(self):
        self.controllerPath = os.path.join(self.appPath, self.controllerFolder)
        if not os.path.isdir(self.controllerPath):
            raise ShimehariSetupError('Controller in the specified directory does not exist.')

        try:
            ctrlDir = self.appFolder + '.' + self.controllerFolder
            ctrlMod = __import__(ctrlDir)
            ctrls = getModulesFromPyFile(self.controllerPath, self.rootPath)
        except (ImportError, AttributeError), error:
            raise ShimehariSetupError('setup controller was failed... \n%s' % error)



    u"""-----------------------------
        ::pkg:: Shimehari.app
        setupBindRouter
        ~~~~~~~~~~~~~~~

        ルーターをバインドします

    ------------------------------"""
    def setupBindRouter(self):
        try:
            routerFile = self.appFolder + '.' + 'router'
            routerMod = __import__(routerFile, fromlist=['router'])
            if hasattr(routerMod, 'appRoutes'):
                self.router = routerMod.appRoutes
        except (ImportError, AttributeError), e:
            raise ShimehariSetupError('Failed to setup the router ...\n details::\n%s' % e)



    u"""-----------------------------
        ::pkg:: Shimehari.app
        beforeRequest
        ~~~~~~~~~~~~~

        リクエスト処理を行う前にする処理を
        登録します。
        [args]
        :f
            実行したい処理
        [return]
        :function
            登録した処理
    ------------------------------"""
    @setupMethod
    def beforeRequest(self, f):
        self.beforeRequestFuncs.setdefault(None,[]).append(f)
        return f



    u"""-----------------------------
        ::pkg:: Shimehari.app
        beforeFirstRequest
        ~~~~~~~~~~~~~~~~~~

        最初のリクエスト処理を行う前にする処理
        [args]
        :f
            実行したい処理
    ------------------------------"""
    @setupMethod
    def beforeFirstRequest(self, f):
        self.beforeFirstRequestFuncs.append(f)



    u"""-----------------------------
        ::pkg:: Shimehari.app
        afterRequest
        ~~~~~~~~~~~~

        リクエストの後に処理
        [args]
        :f
            実行したい処理
    ------------------------------"""
    @setupMethod
    def afterRequest(self, f):
        self.afterRequestFuncs.setdefault(None,[]).append(f)
        return f



    u"""-----------------------------
        ::pkg:: Shimehari.app
        urlValuePreprocessor
        ~~~~~~~~~~~~~~~~~~~~

        リクエストの前に処理
        [args]
        :f
            実行したい処理
    ------------------------------"""
    @setupMethod
    def urlValuePreprocessor(self, f):
        self.urlValuePreprocesors.setdefault(None, []).append(f)
        return f



    @setupMethod
    def tearDownAppContext(self, f):
        self.teardownAppContextFuncs.append(f)
        return f


    @setupMethod
    def errorHandler(self, codeOrException):
        def decorator(f):
            self._registerErrorHandler(None, codeOrException, f)
            return f
        return decorator

    @setupMethod
    def _registerErrorHandler(self, key, codeOrException, f):
        if isinstance(codeOrException, HTTPException):
            codeOrException = codeOrException.code
        if isinstance(codeOrException, (int, long)):
            assert codeOrException != 500 or key is None
            self.errorHandlerSpec.setdefault(key, {})[codeOrException] = f
        else:
            self.errorHandlerSpec.setdefault(key, {}).setdefault(None,[]).append((codeOrException, f))


    u"""-----------------------------
        ::pkg:: Shimehari.app
        tryTriggerBeforeFirstRequest
        ~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        最初のリクエストがあった時のみに行う処理を実行します。
    ------------------------------"""
    def tryTriggerBeforeFirstRequest(self):
        if self._gotFirstRequest:
            return 
        with self._beforeRequestLock:
            if self._gotFirstRequest:
                return 
            self._gotFirstRequest = True
            [f() for f in self.beforeFirstRequestFuncs]



    u"""-----------------------------
        ::pkg:: Shimehari.app
        createAdapter
        ~~~~~~~~~~~~~

        url adapter を生成します
        [args]
        :request
            元となるリクエスト
        [return]
        :adapter
            アダプター
    ------------------------------"""
    def createAdapter(self, request):
        if request is not None:
            return self.router.bind_to_environ(request.environ)
        #なんのこっちゃ
        if self.config['SERVER_NAME'] is not None:
            return self.router.bind(
                self.config['SERVER_NAME'], 
                script_name=self.config['APP_ROOT'] or '/',
                url_scheme=self.config['PREFERRED_URL_SCHEME'])



    u"""-----------------------------
        ::pkg:: Shimehari.app
        appContext
        ~~~~~~~~~~
        アプリケーションコンテキストを返します。
        [args]
        :environ
            環境変数
    ------------------------------"""
    def appContext(self):
        return AppContext(self)



    u"""-----------------------------
        ::pkg:: Shimehari.app
        requestContext
        ~~~~~~~~~~~~~~
        リクエストコンテキストを返します。
        [args]
        :environ
            リクエスト環境変数
    ------------------------------"""
    def requestContext(self,environ):
        return RequestContext(self, environ)



    u"""-----------------------------
        ::pkg:: Shimehari.app
        doAppContextTearDonw
        ~~~~~~~~~~~~~~~~~~~~
        アプリケーションコンテキストがティアーダウンしたとき実行します。
        [args]
        :exc
            
    ------------------------------"""
    def doAppContextTearDonw(self,exc=None):
        if exc is None:
            exc = sys.exc_info()[1]
        for func in reversed(self.teardownAppContextFuncs):
            func(exc)
        appContextTearingDown.send(self, exc)



    u"""-----------------------------
        ::pkg:: Shimehari.app
        doRequestContextTearDown
        ~~~~~~~~~~~~~~
        リクエストコンテキストを返します。
        未実装
        [args]
        :environ
            環境変数
    ------------------------------"""
    def doRequestContextTearDown(self,exc=None):
        if exc is None:
            exc = sys.exc_info()[1]
        for func in reversed(self.teardownRequestContextFuncs.get(None,())):
            rv = func(exc)
        requestContextTearingDown.send(self,exc=exc)



    u"""-----------------------------
        ::pkg:: Shimehari.app
        preprocessRequest
        ~~~~~~~~~~~~~~~~~

        リクエスト前に実行したい処理たちを実行します。
        [return]
        :rv
            登録した処理
    ------------------------------"""
    def preprocessRequest(self):
        for func in self.urlValuePreprocesors.get(None, ()):
            func(request.endpoint, request.viewArgs) 
        self.csrf.checkCSRFExempt()
        self.csrf.csrfProtect()
        for func in self.beforeRequestFuncs.get(None, ()):
            rv = func()
            if rv is not None:                
                return rv



    u"""-----------------------------
        ::pkg:: Shimehari.app
        processResponse
        ~~~~~~~~~~~~~~~
        レスポンス前に行いたい処理を実行します。
        [return]
        :response
            レスポンス
    ------------------------------"""
    def processResponse(self,response):
        context = _requestContextStack.top
        funcs = ()

        if None in self.afterRequestFuncs:
            funcs = self.afterRequestFuncs[None]
        for handler in funcs:
            response = handler(response)
        
        if not self.sessionStore.isNullSession(context.session):
            self.saveSession(context.session, response)
        return response



    u"""-----------------------------
        ::pkg:: Shimehari.app
        dispatchRequest
        ~~~~~~~~~~~~~~~
        リクエストだす
        [args]
        :request
            リクエスト

        [return]
        :response
            レスポンス
    ------------------------------"""
    def dispatchRequest(self):
        self.tryTriggerBeforeFirstRequest()
        try:
            rv = self.preprocessRequest()
            if rv is None:
                req = _requestContextStack.top.request
                if req.routingException is not None:
                    self.raiseRoutingException(req)
                rule = req.urlRule
                rv = self.controllers[rule.endpoint](**req.viewArgs)
        except Exception, e:
            rv = self.makeResponse(self.handleUserException(e))

        response = self.makeResponse(rv)
        response = self.processResponse(response)
        
        return response



    u"""-----------------------------
        ::pkg:: Shimehari.app
        makeResponse
        ~~~~~~~~~~~~
        レスポンスを生成して返します。
        [args]
        :rv
            リクエスト
        [return]
        :rv
            レスポンス
    ------------------------------"""
    def makeResponse(self, rv):
        status = headers = None

        if isinstance(rv,tuple):
            rv, status, headers = rv + (None,) * (3-len(rv))

        if rv is None:
            raise ValueError('view function does not return a response.')

        if not isinstance(rv, self.responseClass):
            if isinstance(rv, basestring):
                rv = self.responseClass(rv, headers=headers, status=status)
                headers = status = None
            else:
                rv = self.responseClass.force_type(rv,request.environ)
        if status is not None:
            if isinstance(status, basestring):
                rv.status = status
            else:
                rv.status_code = status
        if headers:
            rv.headers.extend(headers)
        return rv



    u"""-----------------------------
        ::pkg:: Shimehari.app
        handleException
        ~~~~~~~~~~~~~~~
        エラーをハンドリングします。

        [args]
        :e
            エラー内容
        [return]

    ------------------------------"""
    def handleException(self, e):

        excType, excValue, excTb = sys.exc_info()
        handler = self.errorHandlerSpec[None].get(500)

        if self.propagateExceptions:
            if excValue is e:
                raise excType, excValue, excTb
            else:
                raise e

        self.logException((excType,excValue,excTb))
        if handler is None:
            return InternalServerError()
        return handler(e)



    def handleHTTPException(self, e):
        handler = self.errorHandlerSpec[None].get(e.code)
        if handler is None:
            return e
        return handler(e)


    def trapHTTPException(self, e):
        if self.config['TRAP_HTTP_EXCEPTIONS']:
            return True
        if self.config['TRAP_BAD_REQUEST_ERRORS']:
            return isinstance(e, BadRequest)
        return False

    def handleUserException(self,e):
        excType, excValue, tb = sys.exc_info()
        assert excValue is e
        
        if isinstance(e, HTTPException) and not self.trapHTTPException(e):
            return self.handleHTTPException(e)

        appHandlers = self.errorHandlerSpec[None].get(None, ())
        for typecheck, handler in appHandlers:
            if isinstance(e, typecheck):
                return handler(e)
        raise excType, excValue, tb


    def raiseRoutingException(self, request):
        if not self.debug or not isinstance(request.routingException, RequestRedirect) \
           or request.method in ('GET', 'HEAD', 'OPTIONS'):
            raise request.routingException



    def testRequestContext(self, *args, **kwargs):
        from shimehari.testing import makeTestEnvironBuilder
        builder = makeTestEnvironBuilder(self, *args, **kwargs)
        try:
            return self.requestContext(builder.get_environ())
        finally:
            builder.close()

    def handleBuildError(self, error, endpoint, **kwargs):
        if self.buildErrorHandlers is None:
            excType, excValue, tb = sys.exc_info()
            if excValue is error :
                raise excType, excValue, tb
            else:
                raise error
        return self.buildErrorHandlers(error, endpoint, **kwargs)




    u"""-----------------------------
        ::pkg:: Shimehari.app
        wsgiApp
        ~~~~~~~

        WSGI アプリとして実行します。(であってるのか)
        [args]
        :environ
            環境変数
        :startResponse
            hoge
    ------------------------------"""
    def wsgiApp(self, environ, startResponse):
        with self.requestContext(environ):
            try:
                response = self.dispatchRequest()
            except Exception, e:
                response = self.makeResponse(self.handleException(e))
            return response(environ, startResponse)
        


    u"""--------------------------------------
        ::pkg:: Shimehari.app
        drink
        ~~~~~    

        アプリを走らせます。
        [args]
        :host
            ホスト名
        :port
            ポート番号
        :debug
            デバッグモードとして起動するかどうか
        :options
            kwargs
    --------------------------------------"""
    def drink(self, host=None, port=None, debug=None, **options):
        from werkzeug.serving import run_simple
        if host is None:
            host = '127.0.0.1'
        if port is None:
            port = 5959
        if debug is not None:
            self.debug = bool(debug)
        options.setdefault('use_reloader', self.debug)
        options.setdefault('use_debugger', self.debug)
        try:
            from werkzeug._internal import _log
            _log('info',' * Shimehari GKGK!')
            run_simple(host, port, self, **options)
        finally:
            self._gotFirstRequest = False



    u"""--------------------------------------
        ::pkg:: Shimenari.app
        run
        ~~~

        アプリを走らせます。
        drink のラッパー。
        WSGI 周りのライブラリとかで
        run を自動的に呼ぶ物対策
    --------------------------------------"""
    def run(self,host=None,port=None, debug=None, **options):
        self.drink(host,port,debug,options)


    def testClient(self, useCookies=True):
        cls = self.testClientCls
        if cls is None:
            from shimehari.testing import ShimehariClient as cls
        return cls(self, self.responseClass, use_cookies=useCookies)



    def __call__(self,environ,startResponse):
        return self.wsgiApp(environ,startResponse)



    def __str__(self):
        return 'Shimehari WSGI Application Framework!'




#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
	Shimehari.controllers
	~~~~~~~~~~~~~~~~~~~~~
	コントローラーたち
===============================
"""
from abc import ABCMeta, abstractmethod

u"""
===============================
	Shimehari.controllers.AbstractController
	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	コントローラーの抽象クラス
===============================
"""
class AbstractController(object):

	__metaclass__ = ABCMeta

	baseName = ''

	def __init__(self,name):
		if not name:
			name = self.__class__.__name__
		self.name = name	
		if 'Controller' in name:
			name = name.replace('Controller', '')
		
		self.baseName = name.lower()
	


	@abstractmethod
	def httpMethodNotAllowed(self,templateName='405.html'):
		return



u"""
===============================
	Shimehari.controllers.TemplateRenderableMixIn
	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	テンプレートレンダラブルみっくすいん
===============================
"""
class TemplateRenderableMixIn(object):

	def renderToResponse(self, context, **resKwargs):

		return

	def getTemplateName(self):
		if self.templateName is None:
			raise #なんちゃかえらー
		else:
			return self.templateName




u"""
===============================
	Shimehari.controllers.ApplicationController
	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	コントローラーの具象クラス
===============================
"""
class ApplicationController(TemplateRenderableMixIn, AbstractController):
	def __init__(self,name):
		AbstractController.__init__(self,name)

	def renderTemplate(self,templateName):
		return

	def httpMethodNotAllowed(self,templateName='405.html'):
		pass




#!/usr/bin/env python
# -*- cofing: utf-8 -*-


from werkzeug.exceptions import abort
from werkzeug.utils import redirect

from .app import Shimehari
from .controllers import ApplicationController
from .helpers import jsonAvailable, json, urlFor, jsonify, sendFile
from .routing import Router, Resource, RESTfulRule
from .shared import request, session, shared, _requestContextStack, _appContextStack, currentApp
from .template import renderTemplate, renderTempalteString
from .wrappers import Request, Response

VERSION = '0.1.3'
AUTHOR = 'Yamaguchi Eikichi'


def getVersion():
    return VERSION


#!/usr/bin/env python
# -*- coding: utf-8 -*-

u"""
===============================
    Shimehari.core.config
    ~~~~~~~~~~~~~~~~~~~~~
    shimehari core config settings 
===============================
"""

from werkzeug.datastructures import ImmutableDict

u"""
    許可する RESTful アクション
"""
RESTFUL_ACTIONS = set(['index', 'show', 'edit', 'new', 'create', 'update', 'destroy'])

u"""
    許可する HTTP メソッド
"""
ALLOWED_HTTP_METHOD_NAMES = set(['get', 'post', 'put', 'delete', 'head', 'options', 'trace'])

u"""
    RESTful アクションごとに許可する HTTP メソッドの対応マップ
"""
RESTFUL_METHODS_MAP = ImmutableDict({
    'index':['get'],
    'show':['get','post'],
    'edit':['get','post'],
    'new':['get'],
    'create':['post'],
    'update':['put'],
    'destroy':['delete']
    })



# -*- coding: utf-8 -*-
#!/usr/bin/env python

u"""
===============================
    Shimehari.logging
    ~~~~~~~~~~~~~~~~~
    ロガー
    Flask などなど参考に。
    ルーティングが多少複雑になっても
    対応できるような作りにしたいなぁ
===============================
"""

from __future__ import absolute_import

from logging import getLogger, StreamHandler, Formatter, getLoggerClass, DEBUG

def createLogger(app):

    Logger = getLoggerClass()

    class DebugLogger(Logger):
        def getEffectiveLevel(x):
            if x.level == 0 and app.debug:
                return DEBUG
            return Logger.getEffectiveLevel(x)

    class DebugHandler(StreamHandler):
        def emit(x,record):
            StreamHandler.emit(x,record) if app.debug else None

    handler = DebugHandler()
    handler.setLevel(DEBUG)
    handler.setFormatter(Formatter(app.debugLogFormat))
    logger = getLogger(app.loggerName)

    del logger.handlers[:]
    logger.__class__ = DebugLogger
    logger.addHandler(handler)
    return logger